//******************************************************************************

// @File         model_ps.glsl

// @Version       1.0

// @Created      2017, 3, 8

// @Description  This file is model_vs.glsl.

// @HISTORY:

//******************************************************************************/


#include "common_ps.glsl"


/* Macros in this shader:
_ALPHATEST_						
_ALPHABLEND_					

NORMAL_ENABLE					
UNLIT_ENABLE					

*/

highp vec3 glb_CalcReflection(highp vec3 N, highp vec3 L)
{
    return 2.0 * dot(N, L) * N  - L;
}

//  Calculate R dot V for specular calculation
highp float glb_CalcRDotV(highp vec3 normal, highp vec3 lightDir, highp vec3 viewDir)
{
    highp float fNDotL = dot(normal, lightDir);
    highp vec3 reflection = normalize((2.0 * fNDotL) * normal - lightDir);
    return XSaturate(dot(reflection, viewDir));
}

// Calculate normal from normalmap
highp vec3 glb_SamplerNormalMap(sampler2D normalSampler, highp vec2 texCoord, highp float smoothNess)
{
    highp vec3 normal;
    normal = 2.0 * texture2D(normalSampler, texCoord).xyz - 1.0;
	normal.z *= smoothNess;
    return normalize(normal);
}

//System parameter
uniform highp vec3	SunLight;
uniform highp vec3	Ambient1;
uniform highp vec3	Ambient2;
uniform highp float g_fDiffusePower;

//-----------------------------------------------------------------
// Parameter
//-----------------------------------------------------------------
uniform highp float		g_smoothNess;
uniform highp float		g_UseReflectMaskSampler;
uniform highp float		g_ReflectParam;

//Textures
uniform sampler2D		g_DiffuseSampler;
uniform sampler2D		g_NormalSampler;
uniform samplerCube 		g_ReflectCubeSampler;
uniform sampler2D	 	g_ReflectMaskSampler;

//Varyings
varying highp vec2	VSOutTexcoord0;
varying highp vec2	VSOutTexcoord1;
varying highp vec3	VSOutNormal0;
varying highp vec3	VSOutViewDir;
varying highp vec3	VSOutWorldViewDir;
varying highp vec3	VSOutLightDir;

// needs tangent and binormal to transform some vectors from tangent space to view space.
varying highp vec3	VSOutTangent;
varying highp vec3	VSOutBinormal;

#ifdef REFLECT_ENABLE
varying highp mat3	VSOutInverseView;
#endif // #if REFLECT_ENABLE

void main (void)
{
    // Normal
#ifdef NORMAL_ENABLE
	highp vec3 normal = glb_SamplerNormalMap(g_NormalSampler, VSOutTexcoord0, g_smoothNess);
#else
	highp vec3 normal = vec3(0,0,1);
#endif
	if (!gl_FrontFacing)
		normal = -normal;

	// Sample the diffuse texture.
	highp vec4 colBaseTex = SampleDiffuseTexture(g_DiffuseSampler, VSOutTexcoord0);

#ifdef _ALPHATEST_
	if (colBaseTex.a < 0.5)
		discard;
#endif // #ifdef _ALPHATEST_

#if defined(_RENDER_QUALITY_LOW_) || defined(UNLIT_ENABLE)

	highp vec3 resultColor = colBaseTex.rgb * 1.1;

#else // #if defined(_RENDER_QUALITY_LOW_) || defined(UNLIT_ENABLE)

	highp vec3 colDiffuse = vec3(0.0, 0.0, 0.0);

	//Light factors
	highp vec3 lightDir = normalize(VSOutLightDir);
	highp vec3 viewDir  = normalize(VSOutViewDir);
	highp vec3 viewReflection = glb_CalcReflection(normal, viewDir);

	highp float fNDotL = dot(normal, lightDir);

	highp float fAmbientLightFactor1 = XSaturate(-fNDotL);
	highp float fAmbientLightFactor2 = 1.0 - XSaturate(abs(fNDotL));
	highp float fNdotV = XSaturate(dot(normal, viewDir));

	highp float fVRDotL = XSaturate(dot(viewReflection, lightDir));

	fNDotL = XSaturate(fNDotL);

	highp vec3 colAmbient = vec3(0.0, 0.0, 0.0);

	//Diffuse
	colDiffuse = SunLight * fNDotL * g_fDiffusePower;

	//Ambient
	colAmbient = Ambient1 * fAmbientLightFactor1 + Ambient2 * fAmbientLightFactor2;

	//Combine Shader
	highp vec3 resultColor = (colDiffuse + colAmbient) * colBaseTex.rgb;
	
#ifdef REFLECT_ENABLE

	highp mat3 matT2C;
	matT2C[0] = VSOutTangent.xyz;
	matT2C[1] = VSOutBinormal.xyz;
	matT2C[2] = cross(VSOutTangent.xyz, VSOutBinormal.xyz);	
	highp vec3 normalworld = normalize(VSOutInverseView * matT2C * normal);	
	
	highp vec3 CubeCoordinates = glb_CalcReflection(normalworld, VSOutWorldViewDir);
	highp vec3 ReflectColor = textureCube(g_ReflectCubeSampler, CubeCoordinates).xyz;
	
	if (g_UseReflectMaskSampler > 0.5)
	{
		ReflectColor *= texture2D(g_ReflectMaskSampler, VSOutTexcoord0).x;
	}
	else
	{
		ReflectColor *= g_ReflectParam;
	}
	
	resultColor += ReflectColor * g_smoothNess;	

#endif // #ifdef REFLECT_ENABLE

#endif // #if defined(_RENDER_QUALITY_LOW_) || defined(UNLIT_ENABLE)

	highp vec4 outputColor = vec4(resultColor, colBaseTex.a);

#ifdef _ALPHABLEND_
	gl_FragColor = outputColor;
#else
	gl_FragColor = vec4(outputColor.rgb, 1.0);
#endif
}
